@@ -8,6 +8,14 @@ import logging
 logger = logging.getLogger(__name__)
 
 
+def to_bool(v):
+    if isinstance(v, bool):
+        return v
+    if isinstance(v, int):
+        return bool(v)
+    return v == "True"
+
+
 class Address(dict):
     columns = [
         "address",  # str, address itself
@@ -15,13 +23,15 @@ class Address(dict):
         "change",  # bool, change or receive
         "label",  # str, address label
         "used",  # bool, does this address have a transaction?
+        "service_id",  # str, reserved for or used by Service.id
     ]
     type_converter = [
         str,
         int,
-        lambda v: v if isinstance(v, bool) else v == "True",
+        to_bool,
+        str,
+        to_bool,
         str,
-        lambda v: v if isinstance(v, bool) else v == "True",
     ]
 
     def __init__(self, rpc, **kwargs):
@@ -41,15 +51,31 @@ class Address(dict):
         self["label"] = label
         self.rpc.setlabel(self.address, label)
 
+    def set_service_id(self, service_id: str):
+        if service_id:
+            # Declares that this Address is associated with a Service
+            self["service_id"] = service_id
+        else:
+            # Frees a reserved Address; reset its label
+            self["service_id"] = None
+            if not self.used and self["label"]:
+                self.set_label("")
+
     @property
     def is_external(self):
         return self.index is None
 
+    @property
+    def is_mine(self):
+        return not self.is_external
+
     @property
     def is_receiving(self):
-        # change can be True, False or None
-        # None means it's external
-        return not self.is_external and not self.change
+        return self.is_mine and not self.change
+
+    @property
+    def is_change(self):
+        return self.is_mine and self.change
 
     @property
     def index(self):
@@ -81,6 +107,14 @@ class Address(dict):
     def is_labeled(self):
         return bool(self["label"])
 
+    @property
+    def is_reserved(self):
+        return bool(self["service_id"])
+
+    @property
+    def service_id(self):
+        return self["service_id"]
+
     def __str__(self):
         return self.address
 
@@ -125,6 +159,7 @@ class AddressList(dict):
         for addr in arr:
             if addr["address"] in self:
                 self[addr["address"]].set_label(addr["label"])
+        self.save()
 
     def add(self, arr, check_rpc=False):
         """
@@ -164,10 +199,14 @@ class AddressList(dict):
                 )
         self.save()
 
-    def set_label(self, address, label):
+    def get_address(self, address: str) -> Address:
         if address not in self:
-            self[address] = self.AddressCls(self.rpc, address=address, label=label)
-        self[address].set_label(label)
+            self[address] = self.AddressCls(self.rpc, address=address)
+        return self[address]
+
+    def set_label(self, address, label):
+        addr_obj = self.get_address(address)
+        addr_obj.set_label(label)
         self.save()
 
     def get_labels(self):
@@ -178,6 +217,29 @@ class AddressList(dict):
                 labels[lbl] = labels.get(lbl, []) + [addr.address]
         return labels
 
+    def associate_with_service(
+        self, address: str, service_id: str, label: str, autosave: bool = True
+    ):
+        """
+        Associates the Address (i.e. sets Address.service_id) with the specified
+        Service.id.
+        """
+        addr_obj = self.get_address(address)
+        addr_obj.set_service_id(service_id)
+        addr_obj.set_label(label)
+        if autosave:
+            self.save()
+
+    def deassociate(self, address: str, autosave: bool = True):
+        """
+        Removes the Address's association with a Service (i.e. sets
+        Address.service_id to None and resets Address.label).
+        """
+        addr_obj = self.get_address(address)
+        addr_obj.set_service_id(None)
+        if autosave:
+            self.save()
+
     def set_used(self, addresses):
         need_save = False
         for address in addresses:
@@ -197,7 +259,11 @@ class AddressList(dict):
         return max(
             0,
             0,
-            *[addr.index for addr in self.values() if addr.change == change],
+            *[
+                addr.index or 0
+                for addr in self.values()
+                if addr.is_mine and addr.change == change
+            ],
         )
 
     def max_used_index(self, change=False):
@@ -205,9 +271,9 @@ class AddressList(dict):
             -1,
             -1,
             *[
-                addr.index
+                addr.index or -1
                 for addr in self.values()
-                if addr.used and addr.change == change
+                if addr.is_mine and addr.used and addr.change == change
             ],
         )
 
